<?php
//
// Scheduled Downtime
// Copyright (c) 2008-2015 Nagios Enterprises, LLC. All rights reserved.
//  
// $Id: status.php 273 2010-08-13 21:28:51Z egalstad $

include_once(dirname(__FILE__) . '/../componenthelper.inc.php');

// Initialization stuff
pre_init();
init_session();

// Grab GET or POST variables and check pre-reqs
grab_request_vars();
check_prereqs();
check_authentication(false);


route_request();


function route_request()
{
    $cmd = grab_request_var("cmd", "");

    switch ($cmd) {
        case "cancel":
            cancel_downtime();
            break;
        case "schedule":
            show_add_downtime();
            break;
        case "submit":
            do_submit_downtime();
            break;
        case "getservices":
            $host = grab_request_var("host", "");
            $args = array('brevity' => 1, 'host_name' => $host, 'orderby' => 'service_description:a');
            $oxml = get_xml_service_objects($args);
            if ($oxml) {
                foreach ($oxml->service as $serviceobj) {
                    $name = strval($serviceobj->service_description);
                    echo "<option value='" . $name . "'>$name</option>\n";
                }
            }
            break;
        default:
            show_downtime();
            break;
    }
}


function show_downtime()
{
    $host_dt = array();
    $service_dt = array();
    $in_effect_count = 0;

    $format = '';
    if (is_null($format = get_user_meta(0, 'date_format'))) { $format = get_option('default_date_format'); }

    $get = '?username='.$_SESSION['username'].'&ticket='.get_user_attr(0, 'backend_ticket').'&query=downtimelist&details=true';
    $url = get_base_url()."includes/components/nagioscore/ui/statusjson.php".$get;

    do_page_start(array("page_title" => _('Scheduled Downtime')), true);
?>

<script type="text/javascript">
function get_datetime_string_dt(t)
{
    var format = parseInt('<?php echo $format; ?>');
    var formatted = '';
    var date = new Date(t);

    if (format == 2) {
        // MM/DD/YYYY HH:MM:SS
        formatted += ('0'+(date.getMonth()+1)).slice(-2)+'/';
        formatted += ('0'+date.getDate()).slice(-2)+'/';
        formatted += date.getFullYear()+' ';
        
    } else if (format == 3) {
        // DD/MM/YYYY HH:MM:SS
        formatted += ('0'+date.getDate()).slice(-2)+'/';
        formatted += ('0'+(date.getMonth()+1)).slice(-2)+'/';
        formatted += date.getFullYear()+' ';
    } else {
        // YYYY-MM-DD HH:MM:SS
        formatted += date.getFullYear()+'-';
        formatted += ('0'+(date.getMonth()+1)).slice(-2)+'-';
        formatted += ('0'+date.getDate()).slice(-2)+' ';
    }

    formatted += ('0'+date.getHours()).slice(-2)+':';
    formatted += ('0'+date.getMinutes()).slice(-2)+':';
    formatted += ('0'+date.getSeconds()).slice(-2);

    return formatted;
}

function get_duration_dt(seconds)
{
    x = seconds;
    seconds = Math.floor(seconds % 60);
    x /= 60;
    minutes = Math.floor(x % 60);
    x /= 60;
    hours = Math.floor(x % 24);
    x /= 24;
    days = Math.floor(x);

    var formatted = '';
    if (days != 0) { formatted += days + 'd '; }
    if (days > 0 || hours != 0 ) { formatted += hours + 'h '; }
    formatted += minutes + 'm ';
    formatted += seconds + 's';

    return formatted;
}

function get_downtimes(async)
{
    $('.refresh').html('<i class="fa fa-spinner fa-14 fa-spin"></i>');
    $.ajax({
        type: "GET",
        async: async,
        url: '<?php echo $url; ?>',
        success: function(dt) {

            var dt_hosts = '';
            var dt_services = '';
            var dt_in_effect = '';
            var dt_hosts_count = 0;
            var dt_services_count = 0;
            var dt_in_effect_count = 0;

            if (Object.keys(dt.data.downtimelist).length > 0) {
                $.each(dt.data.downtimelist, function(k, d) {

                    html = '<tr>';
                    html += '<td>'+d.host_name+'</td>';
                    
                    if (d.service_description != undefined) {
                        html += '<td>'+d.service_description+'</td>';
                    } else {
                        html += '<td>-</td>';
                    }

                    html += '<td>'+get_datetime_string_dt(d.entry_time)+'</td>';
                    html += '<td>'+d.author+'</td>';
                    html += '<td>'+d.comment+'</td>';
                    html += '<td>'+get_datetime_string_dt(d.start_time)+'</td>';
                    html += '<td>'+get_datetime_string_dt(d.end_time)+'</td>';

                    if (d.fixed) {
                        html += '<td><?php echo _("Fixed"); ?></td>';
                    } else {
                        html += '<td><?php echo _("Flexible"); ?></td>';
                    }

                    html += '<td>'+get_duration_dt(d.duration)+'</td>';
                    html += '<td>'+d.downtime_id+'</td>';

                    if (d.triggered_by == 0) { d.triggered_by = 'None'; }
                    html += '<td>'+d.triggered_by+'</td>';

                    html += '<td class="center"><a class="remove dt-tt-bind" data-downtime-type="'+d.type+'" data-downtime-id="'+d.downtime_id+'" data-placement="right" title="<?php echo _("Remove"); ?>"><img src="<?php echo theme_image("cross.png"); ?>"></a></td>';
                    html += '</tr>';

                    if (d.type == 2) {
                        var fx = html.replace('<td>-</td>', '');
                        dt_hosts += fx;
                        dt_hosts_count++;
                    } else if (d.type == 1) {
                        dt_services += html;
                        dt_services_count++;
                    }

                    if (d.is_in_effect) {
                        dt_in_effect += html;
                        dt_in_effect_count++;
                    }

                });
            }
                
            if (dt_hosts == '') { dt_hosts = '<tr><td colspan="11"><?php echo _("No scheduled host downtime."); ?></td></tr>'; }
            if (dt_services == '') { dt_services = '<tr><td colspan="12"><?php echo _("No scheduled services downtime."); ?></td></tr>'; }
            if (dt_in_effect == '') { dt_in_effect = '<tr><td colspan="12"><?php echo _("No downtimes are currently in effect."); ?></td></tr>'; }

            $('#dthosts').html(dt_hosts);
            $('#dtservices').html(dt_services);
            $('#dtin-effect').html(dt_in_effect);
            $('.dt-tt-bind').tooltip();

            $('#host-count').html(dt_hosts_count);
            $('#service-count').html(dt_services_count);
            $('#in-effect-count').html(dt_in_effect_count);

            $('.refresh').html('<img src="<?php echo theme_image("arrow_refresh.png"); ?>">');
            if (async) { $('.message').hide(); }
        }
    });
}

$(document).ready(function() {
    
    get_downtimes(false);
    setInterval(get_downtimes, 15000, true);

    $('.refresh').click(function() {
        get_downtimes(true);
    });

    $('#tabs').on('click', '.remove', function() {
        var downtime_id = $(this).data('downtime-id');
        var downtime_type = $(this).data('downtime-type');
        var typ = 78;

        if (downtime_type == 1) {
            typ = 79;
        }

        var txtHeader = get_language_string("AjaxSubmitCommandHeader");
        var txtMessage = get_language_string("AjaxSubmitCommandMessage");

        var content = "<div id='popup_header'><b>"+txtHeader+"</b></div><div id='popup_data'><p><ul class='ajaxcommandresult'><li class='commandresultwaiting' id='ajaxcommandwaiting'>"+txtMessage+"</li></ul></div>";

        set_child_popup_content(content);
        display_child_popup();

        var args = {
            nsp: '<?php echo get_nagios_session_protector_id(); ?>',
            cmd_typ: typ,
            cmd_mod: 2,
            down_id: downtime_id
        }

        $.post('<?php echo get_base_url()."includes/components/nagioscore/ui/cmd.php"; ?>', args, function(data) {
            $('.commandresultwaiting').each(function() {
                $(this).removeClass("commandresultwaiting");
                $(this).addClass("commandresultok");
                $(this).remove();
            });
            $('ul.ajaxcommandresult').append("<li class='commandresultok'><?php echo _('Command processed successfully.'); ?></li>");
            resize_child_popup();
            fade_child_popup("green");
        });
    });

});

$(function() {
    $('#tabs').tabs().show();
});
</script>

<h1><?php echo _('Scheduled Downtime'); ?></h1>

<?php
if (!empty($_SESSION['msg'])) {
    echo '<div class="message"><ul class="infoMessage" style="margin-top: 0;"><li>'.$_SESSION['msg'].'</li></ul></div>';
    unset($_SESSION['msg']);
}
?>

<div id="tabs" class="hide" style="margin-top: 5px;">
    <ul>
        <li><a href="#hosts"><?php echo _('Hosts'); ?> (<span id="host-count"></span>)</a></li>
        <li><a href="#services"><?php echo _('Services'); ?> (<span id="service-count"></span>)</a></li>
        <li><a href="#in-effect"><?php echo _('In Effect'); ?> (<span id="in-effect-count"></span>)</a></li>
        <a class="tt-bind refresh" style="margin: 5px 0 0 5px; color: #222; display: inline-block;" data-placement="right" title="<?php echo _('Refresh'); ?>"><img src="<?php echo theme_image('arrow_refresh.png'); ?>"></a>
    </ul>

    <div id="hosts">
        <p style="margin: 5px 0;">
            <a href="?cmd=schedule&type=host" class="btn btn-sm btn-primary"><i class="fa fa-clock-o l"></i> <?php echo _('Schedule Host Downtime'); ?></a>
        </p>
        <table class="table table-condensed table-bordered table-striped table-auto-width table-no-margin">
            <thead>
                <tr>
                    <th><?php echo _('Host'); ?></th>
                    <th><?php echo _('Entry Time'); ?></th>
                    <th><?php echo _('Author'); ?></th>
                    <th><?php echo _('Comment'); ?></th>
                    <th><?php echo _('Start Time'); ?></th>
                    <th><?php echo _('End Time'); ?></th>
                    <th><?php echo _('Type'); ?></th>
                    <th><?php echo _('Duration'); ?></th>
                    <th><?php echo _('Downtime ID'); ?></th>
                    <th><?php echo _('Trigger ID'); ?></th>
                    <th class="center"><?php echo _('Actions'); ?></th>
                </tr>
            </thead>
            <tbody id="dthosts">
            </tbody>
        </table>
    </div>

    <div id="services">
        <p style="margin: 5px 0;">
            <a href="?cmd=schedule&type=service" class="btn btn-sm btn-primary"><i class="fa fa-clock-o l"></i> <?php echo _('Schedule Service Downtime'); ?></a>
        </p>
        <table class="table table-condensed table-bordered table-striped table-auto-width table-no-margin">
            <thead>
                <tr>
                    <th><?php echo _('Host'); ?></th>
                    <th><?php echo _('Service'); ?></th>
                    <th><?php echo _('Entry Time'); ?></th>
                    <th><?php echo _('Author'); ?></th>
                    <th><?php echo _('Comment'); ?></th>
                    <th><?php echo _('Start Time'); ?></th>
                    <th><?php echo _('End Time'); ?></th>
                    <th><?php echo _('Type'); ?></th>
                    <th><?php echo _('Duration'); ?></th>
                    <th><?php echo _('Downtime ID'); ?></th>
                    <th><?php echo _('Trigger ID'); ?></th>
                    <th><?php echo _('Actions'); ?></th>
                </tr>
            </thead>
            <tbody id="dtservices">
            </tbody>
        </table>
    </div>

    <div id="in-effect">
        <table class="table table-condensed table-bordered table-striped table-auto-width table-no-margin" style="margin-top: 5px;">
            <thead>
                <tr>
                    <th><?php echo _('Host'); ?></th>
                    <th><?php echo _('Service'); ?></th>
                    <th><?php echo _('Entry Time'); ?></th>
                    <th><?php echo _('Author'); ?></th>
                    <th><?php echo _('Comment'); ?></th>
                    <th><?php echo _('Start Time'); ?></th>
                    <th><?php echo _('End Time'); ?></th>
                    <th><?php echo _('Type'); ?></th>
                    <th><?php echo _('Duration'); ?></th>
                    <th><?php echo _('Downtime ID'); ?></th>
                    <th><?php echo _('Trigger ID'); ?></th>
                    <th><?php echo _('Actions'); ?></th>
                </tr>
            </thead>
            <tbody id="dtin-effect">
            </tbody>
        </table>
    </div>
</div>

<?php
    do_page_end(true);
}


function show_add_downtime($error = '')
{
    $type = grab_request_var('type', 'host');
    $host = grab_request_var('host', '');
    $service = grab_request_var('service', '');
    $com_data = grab_request_var('com_data', '');
    $trigger = grab_request_var('trigger', 0);
    $start_time = grab_request_var('start_time', date("m-d-Y H:i:s", time()));
    $end_time = grab_request_var('end_time', date("m-d-Y H:i:s", strtotime('now + 2 hours')));
    $fixed = grab_request_var('fixed', 1);
    $hours = grab_request_var('hours', 2);
    $minutes = grab_request_var('minutes', 0);
    $childoptions = grab_request_var('childoptions', 0);

    $get = '?username='.$_SESSION['username'].'&ticket='.get_user_attr(0, 'backend_ticket').'&query=downtimelist&details=true';
    $dt_url = get_base_url()."includes/components/nagioscore/ui/statusjson.php".$get;

    do_page_start(array("page_title" => _('Scheduled Downtime')), true);
?>

<script type="text/javascript">
var last_host = '';

function update_service_list()
{
    var host = $('#host-list option:selected').val();
    $.get(base_url + '/includes/components/xicore/downtime.php?cmd=getservices&host='+host, function(data) {
        $('#service-list').html(data);
    });
}

$(document).ready(function() {

    update_service_list();
    last_host = $('#host-list option:selected').val();

    $('#host-list').searchable({maxMultiMatch: 9999});
    $('#service-list').searchable({maxMultiMatch: 9999});

    $('#host-list').change(function() {
        if ($(this).val() != last_host) {
            update_service_list();
            last_host = $(this).val();
        }
    });

    $('.datetimepicker').datetimepicker({
        showOn: 'button',
        buttonImage: '../../../images/datetimepicker.png',
        buttonImageOnly: true,
        dateFormat: 'mm-dd-yy',
        timeFormat: 'HH:mm:ss',
        showHour: true,
        showMinute: true,
        showSecond: true
    });

    $('#fixed').change(function() {
        if ($(this).val() == 0) {
            $('#flexible-box').show();
        } else {
            $('#flexible-box').hide();
        }
    });

    $('form').submit(function() {
        whiteout();
        show_throbber();
    })

});
</script>

<h1><?php echo _('Add Scheduled Downtime'); ?> - <?php echo ucfirst($type); ?></h1>

<?php
if (!empty($error)) {
    echo '<div class="message"><ul class="errorMessage" style="margin-top: 0;"><li><i class="fa fa-exclamation-triangle"></i> <strong>'._('Error').'</strong>: '.$error.'</li></ul></div>';
}
?>

<p>
<?php if ($type == "host") {
    echo _('This command is used to schedule downtime for a particular host. During the specified downtime, Nagios will not send notifications out about the host. When the scheduled downtime expires, Nagios will send out notifications for this host as it normally would. Scheduled downtimes are preserved across program shutdowns and restarts. Both the start and end times should be specified in the following format: mm/dd/yyyy hh:mm:ss. If you select the fixed option, the downtime will be in effect between the start and end times you specify. If you do not select the fixed option, Nagios will treat this as "flexible" downtime. Flexible downtime starts when the host goes down or becomes unreachable (sometime between the start and end times you specified) and lasts as long as the duration of time you enter. The duration fields do not apply for fixed downtime.');
} else {
    echo _('This command is used to schedule downtime for a particular service. During the specified downtime, Nagios will not send notifications out about the service. When the scheduled downtime expires, Nagios will send out notifications for this service as it normally would. Scheduled downtimes are preserved across program shutdowns and restarts. Both the start and end times should be specified in the following format: mm/dd/yyyy hh:mm:ss. If you select the fixed option, the downtime will be in effect between the start and end times you specify. If you do not select the fixed option, Nagios will treat this as "flexible" downtime. Flexible downtime starts when the service enters a non-OK state (sometime between the start and end times you specified) and lasts as long as the duration of time you enter. The duration fields do not apply for fixed downtime.');
}
?>
</p>

<form action="?cmd=submit" method="post">
    <input type="hidden" name="type" value="<?php echo $type; ?>">
    <table class="table table-auto-width table-no-border">
        <tr>
            <td><?php echo _('Host Name'); ?> <i class="fa fa-asterisk fa-req tt-bind" title="<?php echo _('Required'); ?>"></i></td>
            <td>
                <select name="host" id="host-list" class="form-control" style="width: 220px;">
                    <?php
                    $args = array('brevity' => 1, 'orderby' => 'host_name:a');
                    $oxml = get_xml_host_objects($args);
                    if ($oxml) {
                        foreach ($oxml->host as $hostobject) {
                            $name = strval($hostobject->host_name);
                            echo "<option value='" . $name . "' " . is_selected($host, $name) . ">$name</option>\n";
                        }
                    }
                    ?>
                </select>
            </td>
        </tr>
        <tr <?php if ($type == "host") { echo 'class="hide"'; } ?>>
            <td><?php echo _('Service Name'); ?> <i class="fa fa-asterisk fa-req tt-bind" title="<?php echo _('Required'); ?>"></i></td>
            <td>
                <select name="service" id="service-list" class="form-control" style="width: 340px;">
                </select>
            </td>
        </tr>
        <tr>
            <td><?php echo _('Author'); ?> <i class="fa fa-asterisk fa-req tt-bind" title="<?php echo _('Required'); ?>"></i></td>
            <td><input type="text" name="com_author" value="<?php echo get_user_attr(0, 'name'); ?>" class="form-control" readonly></td>
        </tr>
        <tr>
            <td><?php echo _('Comment'); ?> <i class="fa fa-asterisk fa-req tt-bind" title="<?php echo _('Required'); ?>"></i></td>
            <td><input type="text" class="form-control" name="com_data" style="width: 400px;" value="<?php echo $com_data; ?>"></td>
        </tr>
        <tr>
            <td><?php echo _('Triggered By'); ?></td>
            <td>
                <?php
                $ch = curl_init();
                curl_setopt($ch, CURLOPT_URL, $dt_url);
                curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, FALSE);
                curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
                $output = curl_exec($ch);
                curl_close($ch);

                $objs = json_decode($output);
                ?>
                <select class="form-control" id="trigger">
                    <option value="0" selected><?php echo _('None'); ?></option>
                    <?php
                    $options = '';
                    $s = '';
                    $h = '';

                    foreach ($objs->data->downtimelist as $obj) { 
                        if (!empty($obj->service_description)) {
                            $s .= '<option value="' . $obj->downtime_id . '">' . $obj->host_name . ' - '. $obj->service_description .' @ ' . date('m-d-y H:i:s', $obj->start_time/1000) . ' (ID ' . $obj->downtime_id . ')</option>';
                        } else {
                            $h .= '<option value="' . $obj->downtime_id . '">' . $obj->host_name . ' @ ' . date('m-d-Y H:i:s', $obj->start_time/1000) . ' (ID ' . $obj->downtime_id . ')</option>';
                        }
                    }

                    if (!empty($h)) {
                        $options .= '<optgroup label="'._('Host Downtimes').'">' . $h . '</optgroup>';
                    }

                    if (!empty($s)) {
                        $options .= '<optgroup label="'._('Service Downtimes').'">' . $s . '</optgroup>';
                    }

                    echo $options;
                    ?>
                </select>
            </td>
        </tr>
        <tr>
            <td><?php echo _('Type'); ?></td>
            <td>
                <select id="fixed" class="form-control">
                    <option value="1"><?php echo _('Fixed'); ?></option>
                    <option value="0"><?php echo _('Flexible'); ?></option>
                </select>
            </td>
        </tr>
        <tr id="flexible-box" class="hide">
            <td><?php echo _('Duration'); ?></td>
            <td>
                <input type="text" class="form-control" style="width: 40px;" id="flexible-hours" value="2"> Hours
                <input type="text" class="form-control" style="width: 40px; margin-left: 5px;" id="flexible-minutes" value="0"> Minutes
            </td>
        </tr>
        <tr>
            <td><?php echo _("Start Time"); ?> <i class="fa fa-asterisk fa-req tt-bind" title="<?php echo _('Required'); ?>"></i></td>
            <td>
                <input class="form-control datetimepicker req" type="text" id='startdateBox' name="start_time" value="<?php echo $start_time; ?>" size="16">
            </td>
        </tr>
        <tr>
            <td><?php echo _("End Time"); ?> <i class="fa fa-asterisk fa-req tt-bind" title="<?php echo _('Required'); ?>"></i></td>
            <td>
                <input class="form-control datetimepicker req" type="text" id='enddateBox' name="end_time" value="<?php echo $end_time; ?>" size="16">
            </td>
        </tr>
        <tr <?php if ($type != "host") { echo 'class="hide"'; } ?>>
            <td><?php echo _('Child Hosts'); ?></td>
            <td>
                <select id="childoptions" name="childoptions" class="form-control req">
                    <option value="0"><?php echo _('Do nothing with child hosts'); ?></option>
                    <option value="1"><?php echo _('Schedule triggered downtime for all child hosts'); ?></option>
                    <option value="2"><?php echo _('Schedule non-triggered downtime for all child hosts'); ?></option>
                </select>
            </td>
        </tr>
    </table>
    <button type="submit" class="btn btn-sm btn-primary"><?php echo _('Schedule'); ?></button>
    <a href="downtime.php" class="btn btn-sm btn-default"><?php echo _('Cancel'); ?></a>
</form>

<?php
    do_page_end(true);
}


function do_submit_downtime()
{
    $type = grab_request_var('type', 'host');
    $host = grab_request_var('host', '');
    $service = grab_request_var('service', '');
    $com_data = grab_request_var('com_data', '');
    $trigger = grab_request_var('trigger', 0);
    $start_time = grab_request_var('start_time', '');
    $end_time = grab_request_var('end_time', '');
    $fixed = grab_request_var('fixed', 1);
    $hours = grab_request_var('hours', 2);
    $minutes = grab_request_var('minutes', 0);
    $childoptions = grab_request_var('childoptions', 0);

    $cmd_mod = 2;
    $cmd_id = 55;
    if ($type == 'service') {
        $cmd_id = 56;
    }

    // Check for errors
    if (empty($host) || empty($com_data) || empty($start_time) || empty($end_time)) {
        $error = _("You must fill out the entire form.");
        show_add_downtime($error);
        return;
    }

    // Generate post fields
    $args = array("cmd_typ" => $cmd_id,
                  "cmd_mod" => $cmd_mod,
                  "host" => $host,
                  "com_data" => $com_data,
                  "trigger" => $trigger,
                  "start_time" => $start_time,
                  "end_time" => $end_time,
                  "fixed" => $fixed);

    if ($type == 'service') {
        $args['service'] = $service;
    } else {
        $args['childoptions'] = $childoptions;
    }

    if ($fixed == 0) {
        $args['hours'] = $hours;
        $args['minutes'] = $minutes;
    }

    // Submit to backend for processing
    $get = '?username='.$_SESSION['username'].'&ticket='.get_user_attr(0, 'backend_ticket');
    $url = get_base_url()."includes/components/nagioscore/ui/cmd.php".$get;

    $ch = curl_init();
    curl_setopt($ch, CURLOPT_URL, $url);
    curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, FALSE);
    curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
    curl_setopt($ch, CURLOPT_USERAGENT, 'Nagios XI Curl Request');
    curl_setopt($ch, CURLOPT_POST, 1);
    curl_setopt($ch, CURLOPT_POSTFIELDS, $args);
    $output = curl_exec($ch);
    curl_close($ch);

    $msg = '';
    $success = strpos($output, "Your command request was successfully submitted to Nagios for processing.");
    if ($success !== false) {
        $msg = _('Successfully added downtime. May take a little while before it shows up.');
    } else {
        $msg = _('Something went wrong with the commad submitted to Nagios Core.');
    }

    $_SESSION['msg'] = $msg;
    header('Location: downtime.php');
}